@@ -10,26 +10,15 @@ import copy
 import json
 import logging
 import os
-import PyInquirer
+import questionary
 import re
 import subprocess
-import textwrap
 import webbrowser
 
 import nf_core.schema, nf_core.utils
 
 log = logging.getLogger(__name__)
 
-#
-# NOTE: When PyInquirer 1.0.3 is released we can capture keyboard interruptions
-# in a nicer way # with the raise_keyboard_interrupt=True argument in the PyInquirer.prompt() calls
-# It also allows list selections to have a default set.
-#
-# Until then we have workarounds:
-# * Default list item is moved to the top of the list
-# * We manually raise a KeyboardInterrupt if we get None back from a question
-#
-
 
 class Launch(object):
     """ Class to hold config option to launch a pipeline """
@@ -256,11 +245,9 @@ class Launch(object):
             "name": "use_web_gui",
             "message": "Choose launch method",
             "choices": ["Web based", "Command line"],
+            "default": "Web based",
         }
-        answer = PyInquirer.prompt([question])
-        # TODO: use raise_keyboard_interrupt=True when PyInquirer 1.0.3 is released
-        if answer == {}:
-            raise KeyboardInterrupt
+        answer = questionary.unsafe_prompt([question], style=nf_core.utils.nfcore_question_style)
         return answer["use_web_gui"] == "Web based"
 
     def launch_web_gui(self):
@@ -347,14 +334,14 @@ class Launch(object):
         The web builder returns everything as strings.
         Use the functions defined in the cli wizard to convert to the correct types.
         """
-        # Collect pyinquirer objects for each defined input_param
-        pyinquirer_objects = {}
+        # Collect questionary objects for each defined input_param
+        questionary_objects = {}
         for param_id, param_obj in self.schema_obj.schema.get("properties", {}).items():
-            pyinquirer_objects[param_id] = self.single_param_to_pyinquirer(param_id, param_obj, print_help=False)
+            questionary_objects[param_id] = self.single_param_to_questionary(param_id, param_obj, print_help=False)
 
         for d_key, definition in self.schema_obj.schema.get("definitions", {}).items():
             for param_id, param_obj in definition.get("properties", {}).items():
-                pyinquirer_objects[param_id] = self.single_param_to_pyinquirer(param_id, param_obj, print_help=False)
+                questionary_objects[param_id] = self.single_param_to_questionary(param_id, param_obj, print_help=False)
 
         # Go through input params and sanitise
         for params in [self.nxf_flags, self.schema_obj.input_params]:
@@ -364,7 +351,7 @@ class Launch(object):
                     del params[param_id]
                     continue
                 # Run filter function on value
-                filter_func = pyinquirer_objects.get(param_id, {}).get("filter")
+                filter_func = questionary_objects.get(param_id, {}).get("filter")
                 if filter_func is not None:
                     params[param_id] = filter_func(params[param_id])
 
@@ -380,7 +367,7 @@ class Launch(object):
         for param_id, param_obj in self.schema_obj.schema.get("properties", {}).items():
             if not param_obj.get("hidden", False) or self.show_hidden:
                 is_required = param_id in self.schema_obj.schema.get("required", [])
-                answers.update(self.prompt_param(param_id, param_obj, is_required, answers))
+                answers = self.prompt_param(param_id, param_obj, is_required, answers)
 
         # Split answers into core nextflow options and params
         for key, answer in answers.items():
@@ -396,24 +383,26 @@ class Launch(object):
         """Prompt for a single parameter"""
 
         # Print the question
-        question = self.single_param_to_pyinquirer(param_id, param_obj, answers)
-        answer = PyInquirer.prompt([question])
-        # TODO: use raise_keyboard_interrupt=True when PyInquirer 1.0.3 is released
-        if answer == {}:
-            raise KeyboardInterrupt
+        question = self.single_param_to_questionary(param_id, param_obj, answers)
+        answer = questionary.unsafe_prompt([question], style=nf_core.utils.nfcore_question_style)
 
         # If required and got an empty reponse, ask again
         while type(answer[param_id]) is str and answer[param_id].strip() == "" and is_required:
             log.error("'–-{}' is required".format(param_id))
-            answer = PyInquirer.prompt([question])
-            # TODO: use raise_keyboard_interrupt=True when PyInquirer 1.0.3 is released
-            if answer == {}:
-                raise KeyboardInterrupt
+            answer = questionary.unsafe_prompt([question], style=nf_core.utils.nfcore_question_style)
 
-        # Don't return empty answers
+        # Ignore if empty
         if answer[param_id] == "":
-            return {}
-        return answer
+            answer = {}
+
+        # Previously entered something but this time we deleted it
+        if param_id not in answer and param_id in answers:
+            answers.pop(param_id)
+        # Everything else (first time answer no response or normal response)
+        else:
+            answers.update(answer)
+
+        return answers
 
     def prompt_group(self, group_id, group_obj):
         """
@@ -426,29 +415,51 @@ class Launch(object):
         Returns:
           Dict of param_id:val answers
         """
-        question = {
-            "type": "list",
-            "name": group_id,
-            "message": group_obj.get("title", group_id),
-            "choices": ["Continue >>", PyInquirer.Separator()],
-        }
-
-        for param_id, param in group_obj["properties"].items():
-            if not param.get("hidden", False) or self.show_hidden:
-                question["choices"].append(param_id)
-
-        # Skip if all questions hidden
-        if len(question["choices"]) == 2:
-            return {}
-
         while_break = False
         answers = {}
+        error_msgs = []
         while not while_break:
-            self.print_param_header(group_id, group_obj)
-            answer = PyInquirer.prompt([question])
-            # TODO: use raise_keyboard_interrupt=True when PyInquirer 1.0.3 is released
-            if answer == {}:
-                raise KeyboardInterrupt
+
+            if len(error_msgs) == 0:
+                self.print_param_header(group_id, group_obj, True)
+
+            question = {
+                "type": "list",
+                "name": group_id,
+                "qmark": "",
+                "message": "",
+                "instruction": " ",
+                "choices": ["Continue >>", questionary.Separator()],
+            }
+
+            # Show error messages if we have any
+            for msg in error_msgs:
+                question["choices"].append(
+                    questionary.Choice(
+                        [("bg:ansiblack fg:ansired bold", " error "), ("fg:ansired", f" - {msg}")], disabled=True
+                    )
+                )
+            error_msgs = []
+
+            for param_id, param in group_obj["properties"].items():
+                if not param.get("hidden", False) or self.show_hidden:
+                    q_title = [("", "{}  ".format(param_id))]
+                    # If already filled in, show value
+                    if param_id in answers and answers.get(param_id) != param.get("default"):
+                        q_title.append(("class:choice-default-changed", "[{}]".format(answers[param_id])))
+                    # If the schema has a default, show default
+                    elif "default" in param:
+                        q_title.append(("class:choice-default", "[{}]".format(param["default"])))
+                    # Show that it's required if not filled in and no default
+                    elif param_id in group_obj.get("required", []):
+                        q_title.append(("class:choice-required", "(required)"))
+                    question["choices"].append(questionary.Choice(title=q_title, value=param_id))
+
+            # Skip if all questions hidden
+            if len(question["choices"]) == 2:
+                return {}
+
+            answer = questionary.unsafe_prompt([question], style=nf_core.utils.nfcore_question_style)
             if answer[group_id] == "Continue >>":
                 while_break = True
                 # Check if there are any required parameters that don't have answers
@@ -456,17 +467,17 @@ class Launch(object):
                     req_default = self.schema_obj.input_params.get(p_required, "")
                     req_answer = answers.get(p_required, "")
                     if req_default == "" and req_answer == "":
-                        log.error("'{}' is required.".format(p_required))
+                        error_msgs.append(f"`{p_required}` is required")
                         while_break = False
             else:
                 param_id = answer[group_id]
                 is_required = param_id in group_obj.get("required", [])
-                answers.update(self.prompt_param(param_id, group_obj["properties"][param_id], is_required, answers))
+                answers = self.prompt_param(param_id, group_obj["properties"][param_id], is_required, answers)
 
         return answers
 
-    def single_param_to_pyinquirer(self, param_id, param_obj, answers=None, print_help=True):
-        """Convert a JSONSchema param to a PyInquirer question
+    def single_param_to_questionary(self, param_id, param_obj, answers=None, print_help=True):
+        """Convert a JSONSchema param to a Questionary question
 
         Args:
           param_id: Parameter ID (string)
@@ -475,12 +486,12 @@ class Launch(object):
           print_help: If description and help_text should be printed (bool)
 
         Returns:
-          Single PyInquirer dict, to be appended to questions list
+          Single Questionary dict, to be appended to questions list
         """
         if answers is None:
             answers = {}
 
-        question = {"type": "input", "name": param_id, "message": param_id}
+        question = {"type": "input", "name": param_id, "message": ""}
 
         # Print the name, description & help text
         if print_help:
@@ -530,7 +541,11 @@ class Launch(object):
                 try:
                     if val.strip() == "":
                         return True
-                    float(val)
+                    fval = float(val)
+                    if "minimum" in param_obj and fval < float(param_obj["minimum"]):
+                        return "Must be greater than or equal to {}".format(param_obj["minimum"])
+                    if "maximum" in param_obj and fval > float(param_obj["maximum"]):
+                        return "Must be less than or equal to {}".format(param_obj["maximum"])
                 except ValueError:
                     return "Must be a number"
                 else:
@@ -568,46 +583,11 @@ class Launch(object):
 
             question["filter"] = filter_integer
 
-        if param_obj.get("type") == "range":
-            # Validate range type
-            def validate_range(val):
-                try:
-                    if val.strip() == "":
-                        return True
-                    fval = float(val)
-                    if "minimum" in param_obj and fval < float(param_obj["minimum"]):
-                        return "Must be greater than or equal to {}".format(param_obj["minimum"])
-                    if "maximum" in param_obj and fval > float(param_obj["maximum"]):
-                        return "Must be less than or equal to {}".format(param_obj["maximum"])
-                    return True
-                except ValueError:
-                    return "Must be a number"
-
-            question["validate"] = validate_range
-
-            # Filter returned value
-            def filter_range(val):
-                if val.strip() == "":
-                    return ""
-                return float(val)
-
-            question["filter"] = filter_range
-
         if "enum" in param_obj:
             # Use a selection list instead of free text input
             question["type"] = "list"
             question["choices"] = param_obj["enum"]
 
-            # Validate enum from schema
-            def validate_enum(val):
-                if val == "":
-                    return True
-                if val in param_obj["enum"]:
-                    return True
-                return "Must be one of: {}".format(", ".join(param_obj["enum"]))
-
-            question["validate"] = validate_enum
-
         # Validate pattern from schema
         if "pattern" in param_obj:
 
@@ -620,36 +600,22 @@ class Launch(object):
 
             question["validate"] = validate_pattern
 
-        # WORKAROUND - PyInquirer <1.0.3 cannot have a default position in a list
-        # For now, move the default option to the top.
-        # TODO: Delete this code when PyInquirer >=1.0.3 is released.
-        if question["type"] == "list" and "default" in question:
-            try:
-                question["choices"].remove(question["default"])
-                question["choices"].insert(0, question["default"])
-            except ValueError:
-                log.warning(
-                    "Default value `{}` not found in list of choices: {}".format(
-                        question["default"], ", ".join(question["choices"])
-                    )
-                )
-        ### End of workaround code
-
         return question
 
-    def print_param_header(self, param_id, param_obj):
+    def print_param_header(self, param_id, param_obj, is_group=False):
         if "description" not in param_obj and "help_text" not in param_obj:
             return
         console = Console(force_terminal=nf_core.utils.rich_force_colors())
         console.print("\n")
-        console.print(param_obj.get("title", param_id), style="bold")
+        console.print("[bold blue]?[/] [bold on black] {} [/]".format(param_obj.get("title", param_id)))
         if "description" in param_obj:
             md = Markdown(param_obj["description"])
             console.print(md)
         if "help_text" in param_obj:
             help_md = Markdown(param_obj["help_text"].strip())
             console.print(help_md, style="dim")
-            console.print("\n")
+        if is_group:
+            console.print("(Use arrow keys)", style="italic", highlight=False)
 
     def strip_default_params(self):
         """ Strip parameters if they have not changed from the default """
